package aexp.phoneintent;

import android.database.Cursor;
import android.provider.CallLog;
import android.provider.BaseColumns;
import android.content.ContentResolver;
import android.net.ContentURI;
import android.os.Bundle;
import android.content.Intent;
import android.content.IntentReceiver;
import android.content.Context;
import android.util.Log;
import java.util.Map;
import java.util.Set;
import java.util.Iterator;

public class PhoneIntentReceiver extends IntentReceiver {
    private static final String LOG_TAG = "PhoneIntentReceiver";

    public void onReceiveIntent(Context context, Intent intent) {
	  String state = null;
	  StringBuffer buf = new StringBuffer();
	  if( intent.getAction() != null )
	  	buf.append( "Intent action: "+intent.getAction() );
	  if( intent.getCategories() != null ) {
		Set categories = intent.getCategories();
		if( categories != null ) {
		  Iterator it = categories.iterator();
	      buf.append( "; categories: " );
		  int ctr = 0;
		  for( ; it.hasNext() ; ) {
			String category = (String)it.next();
			if( ctr != 0 )
				  buf.append( "/" );
			buf.append( category );
			++ctr;
	      }
        }
	  }
	  if( intent.getData() != null ) {
		buf.append( "; intent data: "+intent.getData().toString() );
	  }
	  Bundle extras = intent.getExtras();
	  if( extras != null ) {
		buf.append( "; extras: " );
		int ctr = 0;
		Map m = extras.getAsMap();
		Set keys = m.keySet();
		for( Iterator it = keys.iterator() ; it.hasNext() ; ) {
		  String key = (String)it.next();
		  Object value = m.get( key );
		  if( ctr != 0 )
				buf.append( "/" );
		  String strvalue = value == null ? 
						"null" : 
						value.toString();
		  if( key.equals( "state" ) )
			state = strvalue;
		  buf.append( key+"="+strvalue );
		  ++ctr;
		}
        Log.i(LOG_TAG, buf.toString() );
		if( "IDLE".equals( state ) ) {
		  Log.i( LOG_TAG,
				"Number of the other party: "+
				getLastCallLogEntry( context ) );
		}
	  }
    }

	private String getLastCallLogEntry( Context context ) {
	  String[] projection = new String[] {
    	BaseColumns._ID,
    	CallLog.Calls.NUMBER,
		CallLog.Calls.TYPE
	  };
	  ContentResolver resolver = context.getContentResolver();
      Cursor cur = resolver.query( 
					CallLog.Calls.CONTENT_URI,
                    projection, 
                    null,
					null,
                    CallLog.Calls.DEFAULT_SORT_ORDER );
      int numberColumn = cur.getColumnIndex( CallLog.Calls.NUMBER ); 
      int typeColumn = cur.getColumnIndex( CallLog.Calls.TYPE );
	  if( !cur.next() ) {
		cur.close();
		return "";
	  }
	  String number = cur.getString( numberColumn );
      String type = cur.getString( typeColumn );
	  String dir = null;
	  try {
		int dircode = Integer.parseInt( type );
		switch( dircode ) {
		  case CallLog.Calls.OUTGOING_TYPE:
				dir = "OUTGOING";
				break;

		  case CallLog.Calls.INCOMING_TYPE:
				dir = "INCOMING";
				break;

		  case CallLog.Calls.MISSED_TYPE:
				dir = "MISSED";
				break;
		}
      } catch( NumberFormatException ex ) {}
	  if( dir == null )
			dir = "Unknown, code: "+type;
	  cur.close();
	  return dir+","+number;
    }

}

